#!/usr/bin/env python3.2
#
# Copyright (c) Net24 Limited, Christchurch, New Zealand 2011-2012
#       and     Voyager Internet Ltd, New Zealand, 2012-2013
#
#    This file is part of py-magcode-core.
#
#    Py-magcode-core is free software: you can redistribute it and/or modify
#    it under the terms of the GNU  General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Py-magcode-core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU  General Public License for more details.
#
#    You should have received a copy of the GNU  General Public License
#    along with py-magcode-core.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Zone instance record.  Maps a zi_id to zone_id
"""


import dns.ttl
from sqlalchemy.orm import relationship
from sqlalchemy.orm.session import object_session 
from sqlalchemy.orm.session import make_transient 

from magcode.core.database import *
from dms.database.zi_update import *
from dms.database.zi_copy import *
from dms.dns import RRTYPE_SOA
from dms.dns import RRTYPE_NS
from dms.dns import new_zone_soa_serial
import dms.database.zone_cfg as zone_cfg
from dms.database.resource_record import RR_SOA
from dms.database.resource_record import RR_NS

@saregister
class ZoneInstance(ZiUpdate, ZiCopy):
    """
    Zone Instance type.
    """
    _table="zone_instances"

    @classmethod
    def _mapper_properties(class_):
        """
        Set up relationship to resource_records table
        """
        zi_table = sql_data['tables'][sql_types['ZoneInstance']]
        rr_table = sql_data['tables'][sql_types['ResourceRecord']]
        rr_comments_table = sql_data['tables'][sql_types['RRComment']]
        zone_sm_type = sql_types['ZoneSM']
        rr_comment_type = sql_types['RRComment']
        rr_type = sql_types['ResourceRecord']
        zone_sm_table = sql_data['tables'][zone_sm_type]
        return {'rrs': relationship(sql_types['ResourceRecord'],
                                passive_deletes=True),
                # No backref given in zone_sm as all_zis is dynamic loading
                'zone':         relationship(zone_sm_type, 
                                primaryjoin=(zone_sm_table.c.get('id')
                                    == zi_table.c.zone_id), viewonly=True),
                'rr_group_comments': relationship(rr_comment_type,
                                primaryjoin=(zi_table.c.get('id')
                                    == rr_table.c.zi_id),
                                secondary=sql_data['tables'][rr_type],
                                secondaryjoin=(rr_table.c.comment_group_id
                                    == rr_comments_table.c.get('id')),
                                viewonly=True),
                'rr_comments': relationship(rr_comment_type,
                                primaryjoin=(zi_table.c.get('id')
                                    == rr_table.c.zi_id),
                                secondary=sql_data['tables'][rr_type],
                                secondaryjoin=(rr_table.c.comment_rr_id
                                    == rr_comments_table.c.get('id')),
                                viewonly=True),
                'apex_comment': relationship(rr_comment_type,
                                    uselist=False,
                                    primaryjoin=(
                                        zi_table.c.apex_comment_group_id 
                                        == rr_comments_table.c.get('id'))),
                }

    def __init__(self, zone_id=None,
            soa_serial=None, soa_refresh=None, soa_retry=None,
            soa_expire=None, soa_minimum=None, soa_mname=None, 
            soa_rname=None, soa_ttl=None, zone_ttl=None, change_by=None 
            ):
        """
        Initialise a Zone Instance row
        """
        self.zone_id = zone_id
        self.soa_serial = soa_serial
        self.soa_refresh = soa_refresh
        self.soa_retry = soa_retry
        self.soa_expire = soa_expire
        self.soa_minimum = soa_minimum
        self.soa_mname = soa_mname
        self.soa_rname = soa_rname
        self.soa_ttl = soa_ttl
        self.zone_ttl = zone_ttl
        self.change_by = change_by
        

    def add_rr(self, rr):
        """
        Add RR to rrs list
        """
        if (not hasattr(self, 'rrs') or not self.rrs):
            self.rrs = []
        self.rrs.append(rr)

    def remove_rr(self, rr):
        """
        Remove RR from self.rrs
        """
        if (not hasattr(self, 'rrs') or not self.rrs):
            self.rrs = []
        self.rrs.remove(rr)

    def get_soa_serial(self):
        """
        Update the SOA serial in the ZI, if it has an SOA (as it should
        when submitted to the update engine.
        """
        # Find SOA record, return if not there.
        rr = [r for r in self.rrs if type(r) == RR_SOA][0]
        serial = rr.get_serial()
        return serial

    def update_soa_serial(self, serial):
        """
        Update the SOA serial in the ZI, if it has an SOA (as it should
        when submitted to the update engine.
        """
        # Find SOA record, return if not there.
        rr = [r for r in self.rrs if type(r) == RR_SOA][0]
        self.soa_serial = serial
        rr.update_serial(serial)

    def update_soa_record(self, db_session):
        """
        Form new SOA record from the information stored in the zi
        """
        if self.zone.use_apex_ns:
            # Read in values from DB global config
            sg = self.zone.sg
            mname = zone_cfg.get_row(db_session, 'soa_mname', sg=sg)
            rname = zone_cfg.get_row(db_session, 'soa_rname', sg=sg)
            refresh = zone_cfg.get_row(db_session, 'soa_refresh', sg=sg)
            retry = zone_cfg.get_row(db_session, 'soa_retry', sg=sg)
            expire = zone_cfg.get_row(db_session, 'soa_expire', sg=sg)
            # Update zi if different
            if mname != self.soa_mname:
                self.soa_mname = mname
            if rname != self.soa_rname:
                self.soa_rname = rname
            if refresh != self.soa_refresh:
                self.soa_refresh = refresh
            if retry != self.soa_retry:
                self.soa_retry = retry
            if expire != self.soa_expire:
                self.soa_expire = expire
            # Set the SOA TTL from zone_ttl so no TTL 'funnies' happen
            # when use_apex_ns is set
            if self.soa_ttl:
                self.soa_ttl = None

        rdata = ("%s %s %s %s %s %s %s" 
                    % (self.soa_mname, self.soa_rname, self.soa_serial,
                            self.soa_refresh, self.soa_retry, self.soa_expire,
                            self.soa_minimum))
        ttl = self.soa_ttl
        zone_ttl = self.zone_ttl
        new_soa_rr = RR_SOA(label='@', ttl=ttl, zone_ttl=zone_ttl,
                rdata=rdata, domain=self.zone.name)
        
        # Put in new SOA record
        # Remove every soa rr but the first. Have to be careful as deleting
        # from lists while looping over them can be problematic, and ordering
        # can influence SQL statement ordering, which could be sensitive here.
        # Being hyper cautious here...
        old_soa_rrs = [r for r in self.rrs if type(r) == RR_SOA]
        if len(old_soa_rrs):
            # Remove every soa rr but the first, then grab comment, then remove
            # thold SOA RR.
            old_soa_rr = old_soa_rrs.pop(0)
            for rr in old_soa_rrs:
                self.remove_rr(rr)
                db_session.delete(rr)
            rr_comment = old_soa_rr.rr_comment
            self.remove_rr(old_soa_rr)
            db_session.delete(old_soa_rr)
        else:
            rr_comment = None
        self.add_rr(new_soa_rr)
        db_session.add(new_soa_rr)
        new_soa_rr.group_comment = self.apex_comment
        new_soa_rr.rr_comment = rr_comment

    def update_apex_ns_records(self, db_session):
        """
        Update the apex NS records
        """
        # Check that Apex NS servers are configured.
        apex_ns_names = zone_cfg.get_rows(db_session, settings['apex_ns_key'],
                                            sg=self.zone.sg)
        if not apex_ns_names:
            log_critical("No Apex NS servers are configured, " 
                                "using current ones")
            return False

        # Locate all apex NS records
        old_apex_ns_rrs = [r for r in self.rrs 
                if (type(r) == RR_NS and r.label == '@')]
        # delete them (seperate from above as delete can affect loop!)
        for rr in old_apex_ns_rrs:
            self.remove_rr(rr)
            db_session.delete(rr)

        # Add new apex NS records from zone_cfg table
        apex_comment = self.apex_comment
        for ns_name in apex_ns_names:
            rr = RR_NS('@', zone_ttl=self.zone_ttl, rdata=ns_name,
                        domain=self.zone.name)
            self.add_rr(rr)
            db_session.add(rr)
            rr.group_comment = apex_comment
        return True
    
    def update_apex(self, db_session, force_apex_ns=False):
        """
        Update Apex SOA and NS records, according to zone_sm.use_apex_ns 
        flag
        """
        self.update_apex_comment(db_session)
        self.update_soa_record(db_session)
        if self.zone.use_apex_ns or force_apex_ns:
            self.update_apex_ns_records(db_session)

    def update_apex_comment(self, db_session):
        """
        Maintain the Zone Apex RRComment
        """
        comment = settings['apex_comment_template'] % self.zone.name
        RRComment = sql_types['RRComment']
        if not self.apex_comment:
            rr_comment = RRComment(comment=comment,
                                    tag=settings['apex_rr_tag'])

            db_session.add(rr_comment)
            self.apex_comment = rr_comment
        else:
            if self.zone.use_apex_ns:
                self.apex_comment.comment = comment
            self.apex_comment.tag = settings['apex_rr_tag']

    def set_apex_comment_text(self, db_session, comment):
        """
        set the text of the Apex Comment
        """
        if not self.apex_comment:
            self.update_apex_comment(db_session)
        if self.zone.use_apex_ns:
            return
        # OK, we can set the apex comment
        self.apex_comment.comment = comment

    def add_apex_comment(self, apex_comment):
        """
        Add the apex comment to the zi
        """
        # only called when constructing zi to save in ZoneEngine._data_to_zi()
        self.apex_comment = apex_comment

    def update_zone_ttls(self, zone_ttl=None, reset_rr_ttl=False):
        """
        Update the zone_ttl across the zi
        """
        if (zone_ttl 
                and dns.ttl.from_text(str(zone_ttl)) 
                    != dns.ttl.from_text(self.zone_ttl)):
            # Only update zi.zone_ttl if it is different - don't
            # surprise people unless it is needed.
            self.zone_ttl = str(zone_ttl)
        for rr in self.rrs:
            rr.update_zone_ttl(self.zone_ttl, reset_rr_ttl)

    def normalize_ttls(self):
        """
        Fixes up ttls in records by finding most common ttl, and blanking the
        ttl field of the resource records with ttl, and setting the zone_ttl
        to the mode. This should find  a value $TTL 
        """
        # Find mode (the hard way!)
        ttl_dict = {}
        for rr in self.rrs:
            rr_ttl = dns.ttl.from_text(rr.ttl) if rr.ttl \
                            else dns.ttl.from_text(rr.zone_ttl) 
            if (rr_ttl in ttl_dict):
                ttl_dict[rr_ttl] += 1
            else:
                ttl_dict[rr_ttl] = 1
        if (not len(ttl_dict)):
            return
        ttl_mode = None
        ttl_max_freq = 0
        for ttl in ttl_dict:
            if ttl_mode is None:
                ttl_mode = ttl
            if ttl_dict[ttl] > ttl_max_freq:
                ttl_mode = ttl
                ttl_max_freq = ttl_dict[ttl]
        self.update_zone_ttls(ttl_mode, reset_rr_ttl=True)
        return

    def iterate_dnspython_rrs(self):
        """
        Iterate through all the dnspython_rr tuples in this zone instance
        """
        for rr in self.rrs:
            if rr.disable:
                # If Record disabled, skip it.
                continue
            yield(tuple(rr.dnspython_rr))

    def to_engine_brief(self, time_format=None):
        """
        Supply data output in brief as a dict.  
        Just zi_id, zone_id, ctime, mtime.
        """
        if not time_format:
            mtime = self.mtime.isoformat(sep=' ') if self.mtime else None
            ctime = self.ctime.isoformat(sep=' ') if self.ctime else None
            ptime = self.ptime.isoformat(sep=' ') if self.ptime else None
        else:
            mtime = self.mtime.strftime(time_format) if self.mtime else None
            ctime = self.ctime.strftime(time_format) if self.ctime else None
            ptime = self.ptime.strftime(time_format) if self.ptime else None

        return {'zi_id': self.id_, 'zone_id': self.zone_id, 
                'ctime': ctime, 'mtime': mtime, 'ptime': ptime, 
                'soa_serial': self.soa_serial, 'change_by': self.change_by}

    def to_engine(self, time_format=None):
        """
        Return all fields as a dict
        """
        if not time_format:
            mtime = self.mtime.isoformat(sep=' ') if self.mtime else None
            ctime = self.ctime.isoformat(sep=' ') if self.ctime else None
            ptime = self.ptime.isoformat(sep=' ') if self.ptime else None
        else:
            mtime = self.mtime.strftime(time_format) if self.mtime else None
            ctime = self.ctime.strftime(time_format) if self.ctime else None
            ptime = self.ptime.strftime(time_format) if self.ptime else None

        return {'zi_id': self.id_, 'zone_id': self.zone_id, 
                'ctime': ctime, 'mtime': mtime, 'ptime': ptime,
                'change_by': self.change_by,
                'soa_serial': self.soa_serial,
                'soa_mname': self.soa_mname,
                'soa_rname': self.soa_rname,
                'soa_refresh': self.soa_refresh,
                'soa_retry': self.soa_retry,
                'soa_expire': self.soa_expire,
                'soa_minimum': self.soa_minimum,
                'soa_ttl': self.soa_ttl,
                'zone_ttl': self.zone_ttl}

    def to_data(self, time_format=None, use_apex_ns=True, all_rrs=False):
        """
        A full zi output with RRs grouped by comment
        """
        # Get given zi as a dict
        result = self.to_engine(time_format=time_format)
        # Get all resource records and group by RR_Group.
        rrs = [rr.to_engine() for rr in self.rrs]
        # Get all the comments, and store them in dicts by id for reference
        rr_group_comments = {}
        for c in self.rr_group_comments:
            # Don't emit comment IDs into JSON
            rr_group_comments[c.id_] = {'comment': c.comment, 'tag': c.tag}
        rr_comments = {}
        for c in self.rr_comments:
            # Don't emit comment IDs into JSON
            rr_comments[c.id_] = {'comment': c.comment, 'tag': c.tag}

        # Header records
        if not all_rrs:
            # Clean out stuff we will not be sending.
            # SOA RR
            rrs = [r for r in rrs if r['type'] != RRTYPE_SOA]
            # apex_ns
            if use_apex_ns:
                rrs = [r for r in rrs
                            if not(r['label'] == '@'
                                and r['type'] == RRTYPE_NS)]

        # RR level commments
        for rr in rrs:
            if rr['comment_rr_id']:
                comment_id = rr['comment_rr_id']
                comment_dict = rr_comments.get(comment_id, None)
                if comment_dict:
                    rr.update(comment_dict)
            del rr['comment_rr_id']

        # Group records by comment_group_id
        rr_groups = {}
        for rr in rrs:
            comment_id = rr['comment_group_id']
            if not comment_id in rr_groups:
                comment_dict = rr_group_comments.get(comment_id, None)
                rr_group = {}
                if comment_dict:
                    rr_group.update(comment_dict)
                rr_group['rrs'] = [rr]
                rr_groups[comment_id] = rr_group
            else:
                rr_groups[comment_id]['rrs'].append(rr)
            # Strip rr comment_group_id
            del rr['comment_group_id']

        result['rr_groups'] = list(rr_groups.values())
        return result


def get_default_zi_data(db_session, sg_name=None):
    """
    Return default zi data from zone_cfg table

    This is called from wsgi code or zone_tool
    """
    zi_data = {}
    soa_fields = ['soa_mname', 'soa_rname', 'soa_refresh', 
            'soa_retry', 'soa_expire', 'soa_minimum']
    for field in soa_fields:
        zi_data[field] = zone_cfg.get_row_exc(db_session, field, 
                sg_name=sg_name)
    zi_data['zone_ttl'] = zone_cfg.get_row_exc(db_session, 'zone_ttl', 
                    sg_name=sg_name)
    zi_data['soa_ttl'] = None
    zi_data['soa_serial'] = new_zone_soa_serial(db_session)
    return zi_data
     

def new_zone_zi(db_session, zone_sm, change_by):
    zi = ZoneInstance(change_by=change_by, 
            **get_default_zi_data(db_session, zone_sm.sg.name))
    db_session.add(zi)
    zi.zone = zone_sm
    zone_sm.all_zis.append(zi)
    db_session.flush()
    # Update SOA and apex NS records
    # Add some NS records to no_use_apex_ns zone so that zone gets
    # into named.
    zi.update_apex(db_session, force_apex_ns=True)
    db_session.flush()
    return zi






#!/usr/bin/env python3.2
#
# Copyright (c) Net24 Limited, Christchurch, New Zealand 2011-2012
#       and     Voyager Internet Ltd, New Zealand, 2012-2013
#
#    This file is part of py-magcode-core.
#
#    Py-magcode-core is free software: you can redistribute it and/or modify
#    it under the terms of the GNU  General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Py-magcode-core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU  General Public License for more details.
#
#    You should have received a copy of the GNU  General Public License
#    along with py-magcode-core.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Reference DB class.
"""


from sqlalchemy.orm import relationship
from sqlalchemy.orm import backref
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.orm.exc import MultipleResultsFound

from magcode.core.database import *
from dms.exceptions import ReferenceExists
from dms.exceptions import ReferenceDoesNotExist
from dms.exceptions import ReferenceStillUsed
from dms.exceptions import MultipleReferencesFound
from dms.exceptions import NoReferenceFound


@saregister
class Reference(object):
    """
    Reference object for tagging data in database with things like customer IDs.
    """
    _table = 'reference'

    @classmethod
    def _mapper_properties(class_):
        zone_sm_type = sql_types['ZoneSM']
        zone_sm_table = sql_data['tables'][zone_sm_type]
        rr_type = sql_types['ResourceRecord']
        return {
                'zones': relationship(zone_sm_type, passive_deletes=True,
                                lazy='dynamic'),
                            }
    
    def __init__(self, reference=None):
        """
        Initialize a reference
        """
        self.reference = reference

    # For comparison purposes, including display!
    def __eq__(self, other):
        return self.reference.lower() == other.reference.lower()

    def __ne__(self, other):
        return self.reference.lower() != other.reference.lower()

    def __lt__(self, other):
        return self.reference.lower() < other.reference.lower()
    
    def __gt__(self, other):
        return self.reference.lower() > other.reference.lower()
    
    def __le__(self, other):
        return self.reference.lower() <= other.reference.lower()
    
    def __ge__(self, other):
        return self.reference.lower() >= other.reference.lower()

    def __str__(self):
        """
        Print out reference name
        """
        return str(self.reference)

    def set_zone(self, zone_sm):
        """
        Set the reference for a zone.

        Uses backref on zone to release old reference if it exists. 
        """
        if hasattr(zone_sm, 'reference') and zone_sm.reference:
            old_ref = zone_sm.reference
            old_ref.zones.remove(zone_sm)
        self.zones.append(zone_sm)
        zone_sm.reference = self

    def to_engine(self, time_format=None):
        """
        Output for zone engine.
        """
        return {'reference_id': self.id_, 'reference': self.reference}

    def to_engine_brief(self, time_format=None):
        """
        Brief output for zone_engine
        """
        return {'reference': self.reference}


def new_reference(db_session, reference, return_existing=False):
    """
    Create a new reference
    """
    ref_obj = Reference(reference)
    try:
        reference_list = db_session.query(Reference)\
                        .filter(Reference.reference.ilike(reference)).all()
        if len(reference_list):
            if not return_existing:
                raise ReferenceExists(reference)
            return reference_list[0]
    except NoResultFound:
        pass
    db_session.add(ref_obj)
    db_session.flush()
    return ref_obj

def del_reference(db_session, reference):
    """
    Delete a reference
    """
    try:
        ref_obj = db_session.query(Reference)\
                        .filter(Reference.reference.ilike(reference)).one()
    except NoResultFound:
        raise ReferenceDoesNotExist(reference)
    # Check that it is no longer being used.
    try:
        zone_sm_type = sql_types['ZoneSM']
        in_use_count = db_session.query(zone_sm_type)\
                        .filter(zone_sm_type.ref_id == ref_obj.id_).count()
        if in_use_count:
            raise ReferenceStillUsed(reference)
    except NoResultFound:
        pass
    db_session.delete(ref_obj)
    db_session.flush()
    del ref_obj

def find_reference(db_session, reference, raise_exc=True):
    """
    Find a reference and return it
    """
    if reference == None:
        if raise_exc:
            raise NoReferenceFound(reference)
        return None
    try:
        ref_obj = db_session.query(Reference)\
                .filter(Reference.reference.ilike(reference)).one()
    except NoResultFound:
        if raise_exc:
            raise NoReferenceFound(reference)
        return None
    except MultipleResultsFound:
        raise MultipleReferencesFound(reference)
    return ref_obj

def rename_reference(db_session, reference, dst_reference):
    """
    Rename a reference 
    """
    try:
        ref_obj = db_session.query(Reference)\
                        .filter(Reference.reference.ilike(reference)).one()
    except NoResultFound:
        raise ReferenceDoesNotExist(reference)
    try:
        reference_list = db_session.query(Reference)\
                        .filter(Reference.reference.ilike(dst_reference)).all()
        if len(reference_list):
            raise ReferenceExists(dst_reference)
    except NoResultFound:
        pass
    ref_obj.reference = dst_reference
    db_session.flush()
    return ref_obj



#!/usr/bin/env python3.2
#
# Copyright (c) Net24 Limited, Christchurch, New Zealand 2011-2012
#       and     Voyager Internet Ltd, New Zealand, 2012-2013
#
#    This file is part of py-magcode-core.
#
#    Py-magcode-core is free software: you can redistribute it and/or modify
#    it under the terms of the GNU  General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Py-magcode-core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU  General Public License for more details.
#
#    You should have received a copy of the GNU  General Public License
#    along with py-magcode-core.  If not, see <http://www.gnu.org/licenses/>.
#
"""
UpdateGroup - collects all the individual RR updates for a zone
"""


from sqlalchemy.orm import relationship

from magcode.core.database import *


@saregister
class UpdateGroup(object):
    """
    Class representing one collective update operation
    """
    _table = "update_groups"

    @classmethod
    def _mapper_properties(class_):
        zone_sm_type = sql_types['ZoneSM']
        zone_sm_table = sql_data['tables'][zone_sm_type]
        rr_type = sql_types['ResourceRecord']
        rr_table = sql_data['tables'][rr_type]
        return {
                'update_ops': relationship(rr_type, passive_deletes=True,
                                order_by=rr_table.c.get('id'), 
                                backref='update_group'),
                            }
    
    def __init__(self, update_type, change_by, ptr_only=False, sectag=None):
        """
        Initialize an update group
        """
        self.update_type = update_type
        self.ptr_only = ptr_only
        self.sectag = sectag
        self.change_by = change_by

def new_update_group(db_session, update_type, zone_sm, change_by=None, 
                    ptr_only=False, sectag=None):
    """
    Create a new update group
    """
    update_group = UpdateGroup(update_type, change_by=change_by, 
                                ptr_only=ptr_only, sectag=sectag)
    db_session.add(update_group)
    zone_sm.update_groups.append(update_group)
    # Get it out there to force early raise of IntegrityError
    db_session.flush()
    return update_group
    




#!/usr/bin/env python3.2
#
# Copyright (c) Net24 Limited, Christchurch, New Zealand 2011-2012
#       and     Voyager Internet Ltd, New Zealand, 2012-2013
#
#    This file is part of py-magcode-core.
#
#    Py-magcode-core is free software: you can redistribute it and/or modify
#    it under the terms of the GNU  General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Py-magcode-core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU  General Public License for more details.
#
#    You should have received a copy of the GNU  General Public License
#    along with py-magcode-core.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Server Group utilities

They are here to avoid import nesting problems
"""


import os
import stat
import pwd

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.orm.exc import MultipleResultsFound
from sqlalchemy.sql import or_
from sqlalchemy.sql import and_
from sqlalchemy.sql import not_

from magcode.core.globals_ import settings
from magcode.core.database import sql_types
from dms.exceptions import SgMultipleResults
from dms.exceptions import NoSgFound
from dms.exceptions import SgExists
from dms.exceptions import SgStillHasZones
from dms.exceptions import SgStillHasServers
from dms.database.master_sm import set_mastersm_replica_sg

def list_all_sgs(db_session):
    """
    Return a list of all SG names
    """
    sg_type = sql_types['ServerGroup']
    sg_names = db_session.query(sg_type.name).all()
    sg_names = [x[0] for x in sg_names] 
    return sg_names

def find_sg_byname(db_session, sg_name, raise_exc=False):
    """
    Find an SG by name
    """
    sg_type = sql_types['ServerGroup']
    query = db_session.query(sg_type)\
            .filter(sg_type.name == sg_name)
    multiple_results = False
    try:
        sg = query.one()
    except NoResultFound:
        sg = None
    except MultipleResultsFound:
        multiple_results = True
    if multiple_results:
        raise SgMultipleResults(sg_name)
    if raise_exc and not sg:
        raise NoSgFound(sg_name)
    return sg

def find_sg_byid(db_session, sg_id, raise_exc=False):
    """
    Find an SG by id
    """
    sg_type = sql_types['ServerGroup']
    query = db_session.query(sg_type)\
            .filter(sg_type.id_ == sg_id)
    try:
        sg = query.one()
    except NoResultFound:
        sg = None
    if raise_exc and not sg:
        raise NoSgFound('*')
    return sg

def new_sg(db_session, sg_name, config_dir=None, address=None, 
        alt_address=None, replica_sg=False):
    """
    Create a new SG
    """
    sg_type = sql_types['ServerGroup']
    try:
        sg_list = db_session.query(sg_type)\
                        .filter(sg_type.name == sg_name).all()
        if len(sg_list):
            raise SgExists(sg_name)
    except NoResultFound:
        pass
    _check_config_dir(config_dir)
    sg = sg_type(sg_name, config_dir, master_address=address, 
            master_alt_address=alt_address)
    db_session.add(sg)
    if replica_sg:
        set_mastersm_replica_sg(db_session, sg)
    db_session.flush()
    return sg

def del_sg(db_session, sg_name):
    """
    Delete an SG
    """
    sg_type = sql_types['ServerGroup']
    # Get the SG from the DB.
    try:
        sg = db_session.query(sg_type)\
                    .filter(sg_type.name == sg_name).one()
    except NoResultFound:
        raise NoSgFound(sg_name)
    # Delete SG.  If it is still in use, SQA will raise an exception
    try:
        zone_sm_type = sql_types['ZoneSM']
        query = db_session.query(zone_sm_type)\
                        .filter(or_(zone_sm_type.sg_id == sg.id_,
                                    zone_sm_type.alt_sg_id == sg.id_))
        query = zone_sm_type.query_is_not_deleted(query)
        in_use_count = query.count()
        if in_use_count:
            raise SgStillHasZones(sg_name)
    except NoResultFound:
        pass
    if len(sg.servers):
        raise SgStillHasServers(sg_name)
    db_session.delete(sg)
    db_session.flush()
    del sg

def rename_sg(db_session, sg_name, new_sg_name):
    """
    Rename an SG
    """
    sg_type = sql_types['ServerGroup']
    # Get the SG from the DB.
    try:
        sg = db_session.query(sg_type)\
                    .filter(sg_type.name == sg_name).one()
    except NoResultFound:
        raise NoSgFound(sg_name)
    # Check that new_sg_name does not exist
    try:
        sg_list = db_session.query(sg_type)\
                        .filter(sg_type.name == new_sg_name).all()
        if len(sg_list):
            raise SgExists(new_sg_name)
    except NoResultFound:
        pass
    # Rename the SG
    sg.name = new_sg_name
    db_session.flush()

def set_sg_master_address(db_session, sg_name, address=None):
    """
    Set the master server address for the SG
    """
    sg_type = sql_types['ServerGroup']
    query = db_session.query(sg_type).filter(sg_type.name == sg_name)
    try:
        sg = query.one()
    except NoResultFound:
        raise NoSgFound(sg_name)
    sg.master_address = address
    db_session.flush()

def set_sg_master_alt_address(db_session, sg_name, address=None):
    """
    Set the alternate master server address for the SG
    """
    sg_type = sql_types['ServerGroup']
    query = db_session.query(sg_type).filter(sg_type.name == sg_name)
    try:
        sg = query.one()
    except NoResultFound:
        raise NoSgFound(sg_name)
    sg.master_alt_address = address
    db_session.flush()

def set_sg_config(db_session, sg_name, config_dir=None):
    """
    Set the config_dir of an SG
    """
    _check_config_dir(config_dir)
    sg_type = sql_types['ServerGroup']
    query = db_session.query(sg_type).filter(sg_type.name == sg_name)
    try:
        sg = query.one()
    except NoResultFound:
        raise NoSgFound(sg_name)
    sg.config_dir = config_dir
    db_session.flush()

def set_sg_replica_sg(db_session, sg_name):
    """
    Set the replica_sg flag
    """
    sg = None
    if sg_name:
        sg_type = sql_types['ServerGroup']
        query = db_session.query(sg_type).filter(sg_type.name == sg_name)
        try:
            sg = query.one()
        except NoResultFound:
            raise NoSgFound(sg_name)
    set_mastersm_replica_sg(db_session, sg)
    db_session.flush()


def _check_config_dir(config_dir):
    """
    Stat a directory and check that it exists, and is readable by 
    dmsdmd user
    """
    if not (config_dir):
        return
    # Check that config_dir exists
    stat_info = os.stat(config_dir)
    # Check that config_dir is a directory
    if not stat.S_ISDIR(stat_info.st_mode):
        raise IOError(errno.ENOTDIR, os.strerror(errno.ENOTDIR))
    # Check that config_dir is readable by run_as_user
    perm_bits = stat.S_IMODE(stat_info.st_mode)
    uid = stat_info.st_uid
    run_as_user = settings.get('run_as_user')
    if (not run_as_user):
        if (not (perm_bits & stat.S_IXOTH and perm_bits & stat.S_IROTH)):
            raise IOError(errno.EACCES, os.strerror(EACCESS))
        return
    try:
        run_as_user_pwd = pwd.getpwnam(run_as_user)
    except KeyError as exc:
        raise IOError(errno.EOWNERDEAD, os.strerror(errno.EOWNERDEAD))

    if (not (perm_bits & stat.S_IXOTH and perm_bits & stat.S_IROTH)
        and not ( uid == run_as_user_pwd.uid and perm_bits &stat.S_IXUSR 
            and perm_bits & stat.SIRUSR)):
        raise IOError(errno.EACCES, os.strerror(EACCESS))
    # If we get here, we can be sure dmsdmd can access the directory
    return



#!/usr/bin/env python3.2
#
# Copyright (c) Net24 Limited, Christchurch, New Zealand 2011-2012
#       and     Voyager Internet Ltd, New Zealand, 2012-2013
#
#    This file is part of py-magcode-core.
#
#    Py-magcode-core is free software: you can redistribute it and/or modify
#    it under the terms of the GNU  General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Py-magcode-core is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU  General Public License for more details.
#
#    You should have received a copy of the GNU  General Public License
#    along with py-magcode-core.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Zone parser pyparsing sandpit module!

Broken out like this for interactive test and debug.
See test section at end of file.

$ORIGIN even though parsed is only for documentation purposes, and $INCLUDE
is not processed in the following. $TTL feeds into the zone_ttl setting for a 
zone.
"""

import re
import sys

from pyparsing import *
import dns.name

from magcode.core.globals_ import *
from dms.globals_ import *
from dms.database.resource_record import rrtype_map


# Comment leader settings
comment_group_leader = settings['comment_group_leader']
comment_rr_leader = settings['comment_rr_leader']
comment_rrflags_leader = settings['comment_rrflags_leader']
comment_anti_regexp = settings['comment_anti_regexp']
rr_flag_lockptr = settings['rr_flag_lockptr']
rr_flag_forcerev = settings['rr_flag_forcerev']
rr_flag_disable = settings['rr_flag_disable']
rr_flag_ref = settings['rr_flag_ref']
rr_flag_rrop = settings['rr_flag_rrop']
rr_flag_trackrev = settings['rr_flag_trackrev']

# global data
previous_label = None
zone_parser_testing = False

def set_test_mode():
    global zone_parser_testing
    zone_parser_testing = True

# Turn of linefeed \n is whitespace as much as possible as zone files are line
# oriented, and turning off \n in pyparsing class data default whitespace 
# in base class will affect parsing of key files elsewhere in program...
# 
# Zone files as per RFC 1034/1035 use the absence of a owner/label
# (blank whitespace) in an RR record to say that the line is addtional 
# record for the last owner/label
# The label/owner and 'blank' have to be hard against the start of line!...


# Parsing exceptions for label, class, type

class ParseLabelException(ParseBaseException):
    """
    If dnspython can't decode this label, throw a stringy!
    """
    pass

class ParseClassException(ParseBaseException):
    """
    If ain't IN, swear to all the dogs in town!
    """
    pass

class ParseTypeException(ParseBaseException):
    """
    If we don't support this RR type, throw another stringy!
    """
    pass

class ParseNoPreviousLabelException(ParseBaseException):
    """
    No previous label found, lets throw a wangy!
    """
    pass

class SetWhitespace(object):
    def __init__(self, whitespacechars):
        self.whitespacechars = whitespacechars
                     
    def __call__(self,pyparsing_expr):
        pyparsing_expr.setWhitespaceChars(self.whitespacechars)
        return pyparsing_expr

# Wrapper calls to make \n not whitespace...
lo = SetWhitespace(' \t\r')
noWS = SetWhitespace('')

rdatachars = alphanums + '/' + '+' + '=' + '.' + ':' + '@' + '-' + '$' + "\\" + '!' + '#' + '%' + '^' + '&' + '*' + '_' + '|' + '{' + '}' + '[' + ']' + "'" + ',' + '?' 
labelchars = alphanums + '.' + '-' +'_' + '*' + '@'
zonechars = alphanums + '.' + '-' +'_'
refchars = alphanums + '.' + '-' + '_' +'@'
rropchars = alphanums + '_'
updatetypechars = alphanums + '_' + '.' + '-'
ttlchars = nums + 'wWdDhHmMsS'
blank = White(ws=' \t')
whiteline = lo(Suppress(lo(blank + LineEnd())))
nullline = LineEnd()
endofdoc = (StringEnd() | (LineEnd()+StringEnd()))
o_paren = Suppress((Literal('(')))
c_paren = Suppress((Literal(')')))
lo_line_end = lo(Suppress(LineEnd()))
lo_line_start = lo(Suppress(LineStart()))
comment_blank_line = lo(Literal(';') + LineEnd())
comment_line = lo(lo(Regex(comment_anti_regexp)) + restOfLine + LineEnd())
comment = lo(Suppress(lo(comment_blank_line|comment_line)))

# Comment text line
comment_txt_ln = noWS(Regex(r'[\S \t]+'))
comment_txt_ln.setParseAction(lambda tokens : tokens[0])
comment_spc = noWS(Suppress(noWS(Literal(' '))))

def crr_parse_action(tokens):
    if not tokens:
        return
    result =  {'comment': '\n'.join(tokens) + '\n'}
    result['type'] = 'comment_rr'
    return result
comment_rr_start = noWS(Suppress(noWS(Combine(comment_rr_leader))))
comment_rr_ln = lo(comment_rr_start + comment_spc + comment_txt_ln + lo_line_end)
comment_rr_blank = lo(comment_rr_start + lo_line_end)
comment_rr_blank.setParseAction(lambda tokens : '')
comment_rr = lo(OneOrMore(lo(comment_rr_ln|comment_rr_blank)))
comment_rr.setParseAction(crr_parse_action)


rrflags_re_start = re.compile('^' + comment_rrflags_leader)
def rrflags_start_fail(s, loc, expr, err):
    if rrflags_re_start.search(s[loc:]):
        # This is a HACK, but it saves stuffing around here. 
        msg = ('Expected "%s|%s|%s|%s|%s<reference>|%s<update-op>"' 
                % (rr_flag_lockptr, rr_flag_forcerev, rr_flag_trackrev, 
                    rr_flag_disable, rr_flag_ref, rr_flag_rrop))
        raise ParseFatalException(err.pstr, err.loc, msg, err.parserElement)
    else:
        raise err
def crrf_parse_action(s, loc, tokens):
    if not tokens:
        return
    result = {'rr_flags': ' '.join(tokens)}
    result['type'] = 'comment_rrflags'
    result['rdata_pyparsing'] = {'s': s, 'loc': loc}
    return result
def crrfr_parse_action(tokens):
    if not tokens:
        return
    return ''.join(tokens)
comment_rrflags_start = noWS(LineStart()) + noWS(Suppress(lo(Combine(comment_rrflags_leader))))
comment_rrflags_LOCKPTR = lo(Combine(rr_flag_lockptr))
comment_rrflags_FORCEREV = lo(Combine(rr_flag_forcerev))
comment_rrflags_TRACKREV = lo(Combine(rr_flag_trackrev))
comment_rrflags_DISABLE = lo(Combine(rr_flag_disable))
comment_rrflags_REF = lo(And([noWS(Combine(rr_flag_ref)), noWS(Word(refchars))]))
comment_rrflags_REF.setParseAction(crrfr_parse_action)
comment_rrflags_RROP = lo(And([noWS(Combine(rr_flag_rrop)), noWS(Word(rropchars))]))
comment_rrflags_RROP.setParseAction(crrfr_parse_action)
comment_rrflags_flags = lo(OneOrMore(lo(comment_rrflags_LOCKPTR|comment_rrflags_FORCEREV|comment_rrflags_TRACKREV|comment_rrflags_DISABLE|comment_rrflags_REF|comment_rrflags_RROP)))
comment_rrflags = comment_rrflags_start + comment_rrflags_flags + lo_line_end
comment_rrflags.setParseAction(crrf_parse_action)
comment_rrflags.setFailAction(rrflags_start_fail)

def rdata_fail_action(s, loc, expr, err):
    # Deal with no Rdata
    if (s[loc] == '\n' or s[loc:err.loc-1].strip() == ''):
        msg = 'Expected RDATA'
        raise ParseFatalException(err.pstr, err.loc-1, msg)
    raise err
def rdata_parse_action(s, loc, tokens):
    if zone_parser_testing:
        return ' '.join(tokens)
    rdata = {}
    rdata['rdata'] = ' '.join(tokens)
    rdata['pyparsing'] = {'s': s, 'loc': loc}
    return rdata
rdata_end = LineEnd()
rdata_comment = lo(lo(Literal(';')) + lo(restOfLine))
rdata_word = lo(Or([dblQuotedString, lo(Word(rdatachars))]))
rdata_word_ml = dblQuotedString | Word(rdatachars)
rdata_1l = lo(And([lo(OneOrMore(rdata_word)), lo(Suppress(rdata_end))]))
rdata_ml = lo(And([lo(ZeroOrMore(rdata_word)), lo(o_paren + OneOrMore(rdata_word_ml) + c_paren), lo(ZeroOrMore(rdata_word)), lo(Suppress(rdata_end))]))
rdata = lo(Or([rdata_ml, rdata_1l]))
rdata.setParseAction(rdata_parse_action)
rdata.setFailAction(rdata_fail_action)


# Both of the following are to make parsing fail fatally in an RR, as they are
# uniquely identified by starting with '^blob.dot.com' or '^<blank>IN'
rr_re_label = re.compile('^[\-a-zA-Z0-9\._@]+\s+')
rr_re_blank = re.compile('^\s+')
def rr_lbl_fail(s, loc, expr, err):
    if rr_re_label.search(s[loc:]):
        raise ParseFatalException(err.pstr, err.loc, err.msg, err.parserElement)
    else:
        raise err
def rr_lbl_parse_action(s, loc, tokens):
    global previous_label
    previous_label = tokens[0][0]
    return tokens
def rr_cont_fail(s, loc, expr, err):
    if rr_re_blank.search(s[loc:]):
        raise ParseFatalException(err.pstr, err.loc, err.msg, err.parserElement)
    else:
        raise err
def rr_cont_parse_action(s, loc, tokens):
    if not previous_label:
        raise ParseNoPreviousLabelException(s, loc, 
                "No previous label in file")
    return tokens
def rr_label_parse_action(s, loc, tokens):
    try:
        thing = dns.name.from_text(tokens[0], None)
    except:
        raise ParseLabelException(s, loc, 'Invalid DNS Label')
    if (tokens[0].find('@') >= 0 and len(tokens[0]) != 1):
        raise ParseLabelException(s, loc, 'Invalid DNS Label')
    if (tokens[0].find('-') == 0):
        raise ParseLabelException(s, loc, 'Invalid DNS Label')
    if (tokens[0].find('.-') >= 0):
        raise ParseLabelException(s, loc, 'Invalid DNS Label')
    return tokens
def rr_type_parse_action(s, loc, tokens):
    if not tokens[0].upper() in rrtype_map.keys():
        raise ParseTypeException(s, loc, "Unsupported RR type '%s'" % tokens[0])
    return str(tokens[0])
def rr_class_parse_action(s, loc, tokens):
    if tokens[0].upper() != 'IN':
        raise ParseTypeException(s, loc, "Unsupported RR class '%s'" % tokens[0])
    return str(tokens[0])
rr_label = noWS(Word(labelchars))
rr_label.setParseAction(rr_label_parse_action)
rr_type = lo(noWS(Word(alphanums)) + FollowedBy(blank))
rr_type.setParseAction(rr_type_parse_action)
rr_class = lo(noWS(lo(CaselessLiteral('IN')|CaselessLiteral('HS')|CaselessLiteral('CH'))) + FollowedBy(blank))
rr_class.setParseAction(rr_class_parse_action)
rr_ttl = lo(noWS(Regex(r'([0-9]+[wWdDhHmMsS]?){1,7}')) + FollowedBy(blank))
rr_ttl.setParseAction(lambda tokens : str(tokens[0]))
rr_lbl = (Group(noWS(LineStart()) + lo(rr_label.setResultsName('label') + lo(Optional(lo(rr_ttl.setResultsName('ttl') + rr_class.setResultsName('class'))|lo(rr_class.setResultsName('class') + rr_ttl.setResultsName('ttl'))|rr_ttl.setResultsName('ttl')|rr_class.setResultsName('class'))) + rr_type.setResultsName('type') + rdata.setResultsName('rdata').ignore(rdata_comment))))
rr_lbl.setFailAction(rr_lbl_fail)
rr_lbl.setParseAction(rr_lbl_parse_action)
# Had to leave lo() of the following And to let 'blank' be recognised
rr_cont = Group(noWS(LineStart()) + blank.setResultsName('blank') + lo(Optional(lo(rr_ttl.setResultsName('ttl') + rr_class.setResultsName('class'))|lo(rr_class.setResultsName('class') + rr_ttl.setResultsName('ttl'))|rr_ttl.setResultsName('ttl')|rr_class.setResultsName('class'))) + rr_type.setResultsName('type') + rdata.setResultsName('rdata').ignore(rdata_comment))
rr_cont.setFailAction(rr_cont_fail)
rr_cont.setParseAction(rr_cont_parse_action)

def cg_parse_action(tokens):
    if not tokens:
        return
    result =  {'comment': '\n'.join(tokens) + '\n'}
    result['type'] = 'comment_group'
    return result
comment_group_start = noWS(Suppress(noWS(Combine(comment_group_leader))))
comment_group_ln = lo(comment_group_start + comment_spc + comment_txt_ln + lo_line_end)
comment_group_blank = lo(comment_group_start + lo_line_end)
comment_group_blank.setParseAction(lambda tokens : '')
comment_group = lo(OneOrMore(lo(comment_group_ln|comment_group_blank)))
comment_group.setParseAction(cg_parse_action)

directive_comment = noWS(Regex(r';[\S \t]*'))
directive_line_end = lo(Suppress(LineEnd())).ignore(directive_comment)

def origin_parse_action(s, loc, tokens):
    if not tokens:
        return
    result = {'origin': tokens[0][0]}
    result['type'] = result['directive'] = '$ORIGIN'
    if not zone_parser_testing:
        result['rdata_pyparsing'] = {'s': s, 'loc': loc}
    return result
origin = Group(lo(Suppress(lo(Combine('$ORIGIN')))) + lo(Word(zonechars)) + directive_line_end)
origin.setParseAction(origin_parse_action)

def dollar_ttl_parse_action(s, loc, tokens):
    if not tokens:
        return
    result = {'ttl': tokens[0][0]}
    result['type'] = result['directive'] = '$TTL'
    if not zone_parser_testing:
        result['rdata_pyparsing'] = {'s': s, 'loc': loc}
    return result
dollar_ttl = Group(lo(Suppress(lo(Combine('$TTL')))) + lo(Word(ttlchars)) + directive_line_end)
dollar_ttl.setParseAction(dollar_ttl_parse_action)

def dollar_include_parse_action(s, loc, tokens):
    if not tokens:
        return
    result = {'filename': tokens[0][0]}
    if len(tokens[0]) > 1:
        result['origin'] = tokens[0][1]
    result['type'] = result['directive'] = '$INCLUDE'
    if not zone_parser_testing:
        result['rdata_pyparsing'] = {'s': s, 'loc': loc}
    return result
dollar_include = Group(lo(Suppress(lo(Combine('$INCLUDE')))) + lo(Word(printables)) +lo(Optional(lo(Word(zonechars)))) + directive_line_end)
dollar_include.setParseAction(dollar_include_parse_action)

def dollar_generate_parse_action(s, loc, tokens):
    if not tokens:
        return
    result = {}
    result['type'] = result['directive'] = '$GENERATE'
    if not zone_parser_testing:
        result['rdata_pyparsing'] = {'s': s, 'loc': loc}
    result['text'] = tokens[0][0]
    return result
dollar_generate = Group(lo(Suppress(lo(Combine('$GENERATE')))) + comment_spc +comment_txt_ln + directive_line_end)
dollar_generate.setParseAction(dollar_generate_parse_action)

def dollar_update_type_parse_action(s, loc, tokens):
    if not tokens:
        return
    result = {}
    result['type'] = result['directive'] = '$UPDATE_TYPE'
    if not zone_parser_testing:
        result['rdata_pyparsing'] = {'s': s, 'loc': loc}
    result['update_type'] = tokens[0][0]
    return result
dollar_update_type = Group(lo(Suppress(lo(Combine('$UPDATE_TYPE')))) + lo(Word(updatetypechars)) + directive_line_end)
dollar_update_type.setParseAction(dollar_update_type_parse_action)

zone_parser = OneOrMore(origin|dollar_ttl|dollar_include|dollar_generate|dollar_update_type|comment_rr|comment_group|comment_rrflags|nullline|rr_lbl|rr_cont).ignore(comment)

# Uncommenting this makes debug painful..... we only 
#
# from dms.zone_parser import zone_parser
#
# into where this grammar definition is used, and ignore all else.
# __all__ = ('zone_parser', )


# Test by using:    from dms.zone_parser import *
#                   set_test_mode()
#                   zone_parser.parseString(<test-string>, parseAll=True)
# where zone_parser can be replaced by other grammar elements
def init_test():
    set_test_mode()

def do_all_tests(start=''):
    for k in test.keys():
        if (str(k) < str(start)):
            continue
        do_test(k)
        input('>>> Press <Enter>')

def do_test(n):
        print( 'test[%s] data:\n' % str(n) + test[n])
        try: 
            print('Parse Output:\n' + repr(zone_parser.parseString(test[n],
                                            parseAll=True)))
        except ParseBaseException as exc:
            print(str(exc))

rdata_test = {}
rdata_test[1] = "16 anathoth.net.\n"
rdata_test[2] = "ns1.anathoth.net. root.anathoth.net (\n 2011060900\n 600 600\n600 600\n )\n"
rdata_test[3] = '16 anathoth.net. "Something curly \" is here"\n'

test = {}
test[1] = "host IN A 192.168.23.4\n  IN MX 16 anathoth.net.\n IN A 192.168.34.56\n"
test[2] = "host IN A 192.168.23.4\n  IN MX 16 anathoth.net.\nhost2 IN A 192.168.34.56\n"
test[3] = "host IN A 192.168.23.4\n  IN SOA ns1.anathoth.net. root.anathoth.net. (\n 2011060800\n 600\n ) 600 600 600\n IN MX 16 anathoth.net.\nhost2 IN A 192.168.34.56\n"
test[4] = "host IN A 192.168.23.4\n  IN SOA ns1.anathoth.net. root.anathoth.net. (\n 2011060800\n 600\n  600 600 600 )    \n IN MX 16 anathoth.net.\nhost2 IN A 192.168.34.56\n\nhost3 IN A 192.168.45.6\n"
test[5] = "host IN A 192.168.23.4\n  IN SOA ns1.anathoth.net. root.anathoth.net. (\n 2011060800\n 600\n 600 600 600 );another comment\n IN MX 16 anathoth.net.\nhost2 IN A 192.168.34.56\n\nhost3 600 A 192.168.45.6; another comment \n"
test[6] = ";# RR Comment 1\nhost IN A 192.168.23.4\n;# RRComment 3\n  IN SOA ns1.anathoth.net. root.anathoth.net. (\n 2011060800\n 600\n 600 600 600 );another comment\n;# Stupid MX Record!\n IN MX 16 anathoth.net.\nhost2 IN A 192.168.34.56\n\nhost3 600 A 192.168.45.6; another comment \n"
test[7] = ";# RR Comment 1\nhost IN A 192.168.23.4\n;# RRComment 3\n  IN SOA ns1.anathoth.net. root.anathoth.net. (\n 2011060800\n 600\n 600 600 600 );another comment\n;# Stupid MX Record!\n IN MX 16 anathoth.net.\nhost2 IN A 192.168.34.56\n\n;|\n;| Test RR Group Comment\n;|\nhost3 600 A 192.168.45.6; another comment \n"
test[8] = ';|\n;| Test RR Group Comment\n;|\nhost3 600 A 192.168.45.6; another comment \n'
test[9] = ';|\n;|\n;| Test RR Group Comment\n;|\nhost3 600 A 192.168.45.6; another comment \n'
test[10] = ';| Some Silly Stuff\n;|\n;| Test RR Group Comment\n;|\nhost3 600 A 192.168.45.6; another comment \n'

test[11] = ';#\n;# Test RR Comment\n;#\nhost3 600 A 192.168.45.6; another comment \n'
test['11a'] = ';#\n;# Test RR Comment\n;# \nhost3 600 A 192.168.45.6; another comment \n'
test[12] = ';|\n;|\n;| Test RR Comment\n;# \nhost3 600 A 192.168.45.6; another comment \n'
test[13] = ';# Some Silly Stuff\n;#   Blah\n;# Test RR Comment\n;#    \nhost3 600 A 192.168.45.6; another comment \n'
test['13a'] = ';# Some Silly Stuff\n;#\n;# Test RR Comment\n;#\nhost3 600 A 192.168.45.6; another comment \n'
#
test[14] = '''

$TTL 99999
$ORIGIN anathoth.net.

; This is a comment
;|
;| Apex records for anathoth.net.
;|
@                          IN  SOA         ( ns1          ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )           
                           IN  NS          ns1
                           IN  NS          ns2

; This is a comment

   


;| Group Comment
;# Website Ip Address
@                          IN  A           203.79.116.183
ns1                        IN  A           203.79.116.183
ns2                        IN  A           210.5.55.246

;| Group Comment
;# Website Ip Address
@                          IN  A           203.79.116.183
host1                        IN  A           203.79.116.183
host2                        IN  A           210.5.55.246

;| Group Comment
;|
;#
@                          IN  A           203.79.116.183
host1                        IN  A           203.79.116.183
host2                        IN  A           210.5.55.246

@                          IN  A           203.79.116.183
host1                        IN  A           203.79.116.183
host2                        IN  A           210.5.55.246

'''

test['14a'] = '''

$TTL 99999
$ORIGIN anathoth.net.

;|
;| Apex records for anathoth.net.
;|
@                          IN  SOA         ( ns1          ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )           
                           IN  NS          ns1
                           IN  NS          ns2

; This is a comment

   


;| Group Comment
;# Website Ip Address
@                          IN  A           203.79.116.183
ns1                        IN  A           203.79.116.183
ns2                        IN  A           210.5.55.246

;| Group Comment
;# Website Ip Address
@                          IN  A           203.79.116.183
host1                        IN  A           203.79.116.183
host2                        IN  A           210.5.55.246

;| Group Comment
;|
;#
@                          IN  A           203.79.116.183
host1                        IN  A           203.79.116.183
host2                        IN  A           210.5.55.246

@                          IN  A           203.79.116.183
host1                        IN  A           203.79.116.183
host2                        IN  A           210.5.55.246

'''
test[15] = """
$TTL 600
$ORIGIN anathoth.net.

;|
;| Apex resource records for anathoth.net. I need a new pair of
;| binoculars for cyber-space dancing!!!
;|
@         2*3                 IN  SOA         ( ns1.anathoth.net. ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )           
                           IN  NS          ns1.anathoth.net.
                           IN  NS          ns2.anathoth.net.
                           IN  NS          ns3.anathoth.net.


;# Website Ip Address
@                          IN  A           203.79.116.183
host                       IN  A           210.5.55.246
ns1                        IN  A           203.79.116.183
;# This is the 2nd name server. It is NOT running.
ns2                        IN  A           210.5.55.246

"""

test[16] = """@         2*3                 IN  SOA         ( ns1.anathoth.net. ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )    \n"""

test[17] = """
$TTL 600
$ORIGIN anathoth.net.

;|
;| Apex resource records for anathoth.net. I need a new pair of
;| binoculars for cyber-space dancing!!!
;|
@         23                 IN  SOA         ( ns1.anathoth.net. ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )           
                           IN  NS          ns1.anathoth.net.
                           IN  NS          ns2.anathoth.net.
                           IN  NS          ns3.anathoth.net.

*

;# Website Ip Address
@                          IN  A           203.79.116.183
host                       IN  A           210.5.55.246
ns1                        IN  A           203.79.116.183
;#3 This is the 2nd name server. It is NOT running.
ns2                        IN  A           210.5.55.246

"""
test[18] = """
$TTL 600
$ORIGIN anathoth.net.

;|
;| Apex resource records for anathoth.net. I need a new pair of
;| binoculars for cyber-space dancing!!!
;|
@         23                 IN  SOA         ( ns1.anathoth.net. ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )           
                           IN  NS          ns1.anathoth.net.
                           IN  NS          ns2.anathoth.net.
                           IN  NS          ns3.anathoth.net.


;# Website Ip Address
@                          IN  A           203.79.116.183
host                       IN  A           210.5.55.246
ns1                        IN  A           203.79.116.183
;# This is the 2nd name server. It is NOT running.
ns2                        I*N  A           210.5.55.246

"""
test[19] = """
$TTL 600
$ORIGIN anathoth.net.

;|
;| Apex resource records for anathoth.net. I need a new pair of
;| binoculars for cyber-space dancing!!!
;|
@         23                 IN  SOA         ( ns1.anathoth.net. ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )           
                           IN  NS          ns1.anathoth.net.
                           IN  NS          ns2.anathoth.net.
                           IN  NS          ns3.anathoth.net.


;# Website Ip Address
@                           IN  A           203.79.116.183
;!LOCKPTR
host                       IN  A           210.5.55.246
ns1                        IN  A           203.79.116.183
;# This is the 2nd name server. It is NOT running.
;!FORCEREV DISABLE REF:Anathoth65     
ns2                        IN  A           210.5.55.246
;!TRACKREV DISABLE REF:Anathoth65     
ns3                        IN  A           210.5.55.247
;!RROP:DELETE     
host12                        IN  ANY       ""

"""
test[20] = """;!BLAH
@         23                 IN  SOA         ( ns1.anathoth.net. ;Master NS
                                             matthewgrant5.gmail.com. ;RP email
                                             2011052603   ;Serial yyyymmddnn
                                             86400        ;Refresh
                                             3600         ;Retry
                                             604800       ;Expire
                                             600          ;Minimum/Ncache
                                             )    \n"""

test[21] = """esxi-bay11.c7000-2-b3           IN      A       172.16.15.33

; ==========================================================
; san infrastructure
; ==========================================================
spa.cx4-120                     IN      A       172.16.8.2
spb.cx4-120                     IN      A       172.16.8.3
;
cx4-120                         IN      A       172.16.8.2
                                IN      A       172.16.8.3
;
vnxe                            IN      A       172.16.8.4
nas.vnxe                        IN      A       172.16.1.2

"""

test[22] = """esxi-bay11.c7000-2-b3           IN      A       172.16.15.33

; ==========================================================
; san infrastructure
; ==========================================================
spa.cx4-120                     IN      A       172.16.8.2
spb.cx4-120                     IN      A       172.16.8.3

;
cx4-120                         IN      A       172.16.8.2
                                IN      A       172.16.8.3
;
vnxe                            IN      A       172.16.8.4
nas.vnxe                        IN      A       172.16.1.2

"""

test[23] = """esxi-bay11.c7000-2-b3           IN      A       172.16.15.33

; ==========================================================
; san infrastructure
; ==========================================================
spa.cx4-120                     IN      A       172.16.8.2
spb.cx4-120                     IN      A       172.16.8.3

;

cx4-120                         IN      A       172.16.8.2
                                IN      A       172.16.8.3
;

vnxe                            IN      A       172.16.8.4
nas.vnxe                        IN      A       172.16.1.2

"""
test[24] = """;
vnxe                            IN      A       172.16.8.4
"""
test[25] = """; -----------------------
; external mail relay servers    
; -----------------------
emr.mail                IN      A       210.5.49.130

; -----------------------
; directory services
; -----------------------
; load balancing address
ldap.dir                IN 300  A       210.5.49.18
                        IN 300  A       210.5.49.19
; offical server names
master.dir              IN 300  A       210.5.49.18
replica.dir             IN 300  A       210.5.49.19

; -----------------------
; isx manager
; -----------------------
manager                 IN      A       210.5.49.2
"""
test[26] = """; -----------------------
; external mail relay servers    
; -----------------------
emr.mail                IN      A       210.5.49.130

; -----------------------
; directory services
; -----------------------
; load balancing address
ldap.dir                300  IN A       210.5.49.18
                        300  IN A       210.5.49.19
; offical server names
master.dir              300  IN A       210.5.49.18
replica.dir             300  IN A       210.5.49.19

; -----------------------
; isx manager
; -----------------------
manager                 IN      A       210.5.49.2
"""

test[28] = """$TTL 24h;
; BIND version named 8.2.2-P5 Fri Mar 17 00:16:04 NZDT 2000
; BIND version root@isx-1.foo.bar.net.nz:/root/bind.8.2.2-P5/src/bin/named
; zone 'foo.bar-test21332.co.nz'   first transfer
; from 210.55.4.13:53 (local 210.55.4.10) using AXFR at Sun Dec 24 00:45:45 2000
$ORIGIN co.nz.
foo.bar-test21332 3600    IN      SOA     drs.registerdirect.net.nz. hostmaster.registerdirect.net.nz. (
                2000122401 3600 900 604800 3600 )
        3600    IN      NS      ns1.blah.net.NZ.
        3600    IN      NS      ns2.blah.net.NZ.
        3600    IN      A       210.55.4.14
        3600    IN      MX      10 mta.blah.net.NZ.
$ORIGIN foo.bar-test21332.CO.NZ.
www     3600    IN      CNAME   foo.bar-test21332.CO.NZ.

$INCLUDE /etc/passwd

$INCLUDE /etc/passwd thing.thing.

$GENERATE blah blah blah

$UPDATE_TYPE OxyPoxyBANG12
"""

test[29] = """internal.anathoth.net. IN DS 18174 7 2 C42492DB9DEF5CA9403D26F175247DFE86D913DA4BEDFC7D629F5E57 D6669FEB
"""

test[30] = """_443._tcp.mx.anathoth.net IN TLSA 1 1 573F1CA26AF1A8B04F87DAD9200F87D886C66E7781A528E9FBD564C8C82BA4A8
"""
